# TODO: Add model to python-commons; implement via attrs
def get_test_result_step_model():
    return {
        'title': None,
        'description': '',
        'outcome': None,
        'duration': 0,
        'steps': [],
        'parameters': {},
        'attachments': [],
        # TODO: Add to python-commons
        # 'started_on': '',
        # 'completed_on': None
    }



class TagType:
    EXTERNAL_ID = 'ExternalId='
    DISPLAY_NAME = 'DisplayName='
    LINKS = 'Links='
    TITLE = 'Title='
    WORK_ITEM_IDS = 'WorkItemIds='
    DESCRIPTION = 'Description='
    LABELS = 'Labels='



from .models.label import get_label_model
from .models.tags import TagType
from .models.url_link import get_url_link_model


def parse_tags(tags):
    parsed_tags = {
        TagType.LINKS: [],
        TagType.LABELS: [],
        TagType.WORK_ITEM_IDS: []
    }

    for tag in tags:
        if TagType.EXTERNAL_ID in tag:
            parsed_tags[TagType.EXTERNAL_ID] = tag[len(TagType.EXTERNAL_ID):]

        elif TagType.DISPLAY_NAME in tag:
            parsed_tags[TagType.DISPLAY_NAME] = tag[len(TagType.DISPLAY_NAME):]

        elif TagType.LINKS in tag:
            parsed_tags[TagType.LINKS].extend(
                parse_links(
                    tag[len(TagType.LINKS):]))

        elif TagType.TITLE in tag:
            parsed_tags[TagType.TITLE] = tag[len(TagType.TITLE):]

        elif TagType.WORK_ITEM_IDS in tag:
            parsed_tags[TagType.WORK_ITEM_IDS].extend(
                parse_massive(
                    tag[len(TagType.WORK_ITEM_IDS):]))

        elif TagType.DESCRIPTION in tag:
            parsed_tags[TagType.DESCRIPTION] = tag[len(TagType.DESCRIPTION):]

        elif TagType.LABELS in tag:
            parsed_tags[TagType.LABELS].extend(
                parse_labels(
                    tag[len(TagType.LABELS):]))

    return parsed_tags


def parse_massive(tag: str):
    return tag.split(',')


def parse_labels(tag):
    parsed_labels = []

    for label in parse_massive(tag):
        parsed_labels.append(get_label_model(label))

    return parsed_labels


def parse_links(tag: str):
    parsed_links = []
    json_links = parse_json(tag)

    if not json_links:
        for link in parse_massive(tag):
            parsed_links.append(get_url_link_model(link))

    if isinstance(json_links, tuple):
        parsed_links.extend(json_links)

    if isinstance(json_links, dict):
        parsed_links.append(json_links)

    return parsed_links


def parse_json(json_string: str):
    try:
        return eval(json_string)
    except Exception:
        return



from .models.option import Option
from .models.tags import TagType
from .scenario_parser import get_scenario_external_id
from .tags_parser import parse_tags


def parse_userdata(userdata):
    if not userdata:
        return

    option = Option()

    if 'tmsUrl' in userdata:
        option.set_url = userdata['tmsUrl']

    if 'tmsPrivateToken' in userdata:
        option.set_private_token = userdata['tmsPrivateToken']

    if 'tmsProjectId' in userdata:
        option.set_project_id = userdata['tmsProjectId']

    if 'tmsConfigurationId' in userdata:
        option.set_configuration_id = userdata['tmsConfigurationId']

    if 'tmsTestRunId' in userdata:
        option.set_test_run_id = userdata['tmsTestRunId']

    if 'tmsTestRunName' in userdata:
        option.set_test_run_name = userdata['tmsTestRunName']

    if 'tmsProxy' in userdata:
        option.set_tms_proxy = userdata['tmsProxy']

    if 'tmsAdapterMode' in userdata:
        option.set_adapter_mode = userdata['tmsAdapterMode']

    if 'tmsConfigFile' in userdata:
        option.set_config_file = userdata['tmsConfigFile']

    if 'tmsCertValidation' in userdata:
        option.set_cert_validation = userdata['tmsCertValidation']

    if 'tmsAutomaticCreationTestCases' in userdata:
        option.set_automatic_creation_test_cases = userdata['tmsAutomaticCreationTestCases']

    return option


def filter_out_scenarios(tests_for_launch, scenarios):
    if tests_for_launch:
        for i in range(len(scenarios)):
            tags = parse_tags(scenarios[i].tags + scenarios[i].feature.tags)
            external_id = tags[TagType.EXTERNAL_ID] if \
                TagType.EXTERNAL_ID in tags and tags[TagType.EXTERNAL_ID] else get_scenario_external_id(scenarios[i])

            if external_id not in tests_for_launch:
                del scenarios[i]

    return scenarios


# TODO: Add to python-commons
def convert_step_to_step(step, nested_steps):
    return {
        'title': step['title'],
        'description': step['description'],
        'steps': nested_steps,
    }


# TODO: Add to python-commons
def convert_step_to_step_result(step, nested_step_results):
    model = {
        'title': step['title'],
        'description': step['description'],
        'duration': step['duration'],
        'outcome': step['outcome'],
        'parameters': step['parameters'],
        'attachments': step['attachments'],
        # TODO: Add to model
        # started_on: started_on,
        # completed_on: completed_on
    }

    if nested_step_results:
        model['step_results'] = nested_step_results

    return model



import testit_python_commons.services as adapter
from testit_python_commons.models.outcome_type import OutcomeType
from testit_python_commons.services import AdapterManager
from testit_python_commons.step import Step

from .models.test_result_step import get_test_result_step_model
from .scenario_parser import (
    parse_scenario,
    parse_status)
from .utils import (
    convert_step_to_step,
    convert_step_to_step_result)


class AdapterListener(object):
    __executable_test = None
    __background_steps_count = 0
    __steps_count = 0

    def __init__(self, adapter_manager: AdapterManager):
        self.__adapter_manager = adapter_manager

    def start_launch(self):
        test_run_id = self.__adapter_manager.get_test_run_id()

        self.__adapter_manager.set_test_run_id(test_run_id)

    def get_tests_for_launch(self):
        return self.__adapter_manager.get_autotests_for_launch()

    def get_scenario(self, scenario):
        self.__executable_test = parse_scenario(scenario)
        self.__background_steps_count = len(scenario.background_steps)
        self.__steps_count = len(scenario.steps)

    def set_scenario(self):
        self.__adapter_manager.write_test(self.__executable_test)

    def get_step_parameters(self, match):
        scope = self.get_scope()

        executable_step = get_test_result_step_model()

        for argument in match.arguments:
            executable_step['description'] += f'{argument.name} = {argument.original} '
            executable_step['parameters'][argument.name] = argument.original

        self.__executable_test[scope].append(executable_step)

    def get_step_result(self, result):
        scope = self.get_scope()
        outcome = parse_status(result.status)

        self.__executable_test[scope][-1]['title'] = result.name
        self.__executable_test[scope][-1]['outcome'] = outcome
        self.__executable_test[scope][-1]['duration'] = round(result.duration * 1000)
        self.__executable_test['duration'] += result.duration * 1000

        # TODO: Add to python-commons
        nested_steps, nested_step_results = Step.get_steps_data()
        executable_step = self.__executable_test[scope][-1]
        # TODO: Fix in python-commons
        result_scope = f'{scope}Results' if scope == 'setUp' else 'stepResults'
        self.__executable_test[scope][-1] = convert_step_to_step(
            executable_step,
            nested_steps)
        self.__executable_test[result_scope].append(
            convert_step_to_step_result(
                executable_step,
                nested_step_results))

        if outcome != OutcomeType.PASSED:
            self.__executable_test['traces'] = result.error_message
            self.__executable_test['outcome'] = outcome
            self.set_scenario()
            return

        if scope == 'setUp':
            self.__background_steps_count -= 1
            return

        self.__steps_count -= 1

        if self.__steps_count == 0:
            self.__executable_test['outcome'] = outcome
            self.set_scenario()

    def get_scope(self):
        if self.__background_steps_count != 0:
            return 'setUp'

        return 'steps'

    @adapter.hookimpl
    def add_link(self, link):
        if self.__executable_test:
            self.__executable_test['resultLinks'].append(link)

    @adapter.hookimpl
    def add_message(self, test_message):
        if self.__executable_test:
            self.__executable_test['message'] = str(test_message)

    @adapter.hookimpl
    def add_attachments(self, attach_paths: list or tuple):
        if self.__executable_test:
            self.__executable_test['attachments'] += self.__adapter_manager.load_attachments(attach_paths)

    @adapter.hookimpl
    def create_attachment(self, body, name: str):
        if self.__executable_test:
            self.__executable_test['attachments'] += self.__adapter_manager.create_attachment(body, name)



# TODO: Add model to python-commons; implement via attrs
def get_url_link_model(label):
    return {
        'url': label,
        # TODO: Make optional in Converter python-commons
        'title': None,
        'type': None,
        'description': None
    }
